package com.ekingwin.bas.cloud.document.service;

import java.util.Map;

import com.ekingwin.bas.cloud.document.DocManagerException;
import com.ekingwin.bas.cloud.document.dto.UploadUserDto;
import com.ekingwin.bas.cloud.document.model.DocModel;
import com.ekingwin.bas.cloud.document.model.FileModel;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;


/**
 * 标准文档操作接口
 * @author chenjb
 *
 */
public interface IDocService {
	public static final String SEARCH_RESULTS = "SearchResults";

	/**
	 * 检入文档
	 * @param user	检入用户
	 * @param docModel	文档模型
	 * @return	检入后，更新信息的文档模型
	 * @throws DocManagerException 
	 */
	public DocModel checkIn(UploadUserDto user, DocModel docModel) throws DocManagerException;

	/**
	 * 根据文档的DID获取指定文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 * @return	文档标准模型
	 */
	public DocModel getDocModelBydID(UploadUserDto user, String dID);

	/**
	 * 根据文档的DOCNAME获取指定文档
	 * @param user	操作用户
	 * @param dName	DOCNAME
	 * @return	文档标准模型
	 */
	public DocModel getDocModelBydName(UploadUserDto user, String dName);

	/**
	 * 根据文档的DOCNAME获取指定文档，只包括基本的信息
	 * @param user	操作用户
	 * @param dID	文档的DID
	 * @return	文档标准模型
	 */
	public DocModel getDocModelBasicBydID(UploadUserDto user, String dID);

	/**
	 * 根据文档的DID获取指定文档的文件
	 * @param user	操作用户
	 * @param dID	文档的DID
	 * @param fileName	文件名
	 * @param fileModel	文件模型
	 * @return	文件模型
	 */
	public FileModel getPrimaryFileBydID(UploadUserDto user, String dID, String fileName, FileModel fileModel);

	/**
	 * 根据文档的DID获取指定文档的文件的web文件（即可以通过web访问）
	 * @param user			操作用户
	 * @param dID			文档的DID
	 * @param fileName		文件名
	 * @param fileModel		文件模型
	 * @return		文件模型
	 */
	public FileModel getWebFileBydID(UploadUserDto user, String dID, String fileName, FileModel fileModel);

	/**
	 * 根据查询条件查询文档
	 * @param user			操作用户
	 * @param queryText		查询条件
	 * @return	查询后的结果
	 */
	public Map<String, Object> searchByQueryText(String userId, String queryText);

	/**
	 * 根据查询条件查询文档，支持分页以及排序
	 * @param user			操作用户
	 * @param queryText		查询条件
	 * @param orderBy		排序字段
	 * @param sort			排序方式
	 * @param rowCount		查询数量
	 * @param startRow		开始条数
	 * @return	查询后的结果
	 */
	public Map<String, Object> searchByQueryText(String userId, String queryText, String orderBy, String sort, Integer rowCount, Integer startRow);

	/**
	 * 根据查询条件查询文档，支持分页以及排序
	 * @param criteria	分页对象
	 * @return	查询后的文档模型集合
	 * @throws DocManagerException 
	 */
	public PageCriteria<DocModel> search(QueryDTO query) throws DocManagerException;

	/**
	 * 检出指定的文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void checkOutBydID(UploadUserDto user, String dID);

	/**
	 * 取消检出指定的文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void unCheckOutBydID(UploadUserDto user, String dID);

	/**
	 * 更新文档
	 * @param user		操作用户
	 * @param docModel	包含更新信息的文档模型
	 * @return			更新后的文档模型
	 */
	public DocModel updataDocModel(UploadUserDto user, DocModel docModel);

	/**
	 * 更新文档到新的版本
	 * @param user		操作用户
	 * @param docModel	包含更新信息的文档模型
	 * @return			更新后的文档模型
	 */
	public DocModel updataNewRevDocModel(UploadUserDto user, DocModel docModel);

	/**
	 * 更新文档的元数据
	 * @param userId	操作用户
	 * @param attrs		需要更新的元数据集合
	 */
	public void updataDocMeda(String userId, Map<String, String> attrs);

	/**
	 * 删除指定文档的所有版本
	 * @param user		操作用户
	 * @param dID		文档的DID
	 * @param docName	文档的DOCNAME
	 */
	public void deleteDocModelAllRevision(UploadUserDto user, String dID, String docName);

	/**
	 * 直接删除文档，无法恢复
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void deleteNoReconver(UploadUserDto user, String dID);

	/**
	 * 根据指定的文档DID删除文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void deleteDocModelBydId(UploadUserDto user, String dID);

	/**
	 * 批量删除文档
	 * @param user		操作用户
	 * @param docids	需要删除的文档DID的集合
	 */
	public void batchDelete(UploadUserDto user, String[] docids);

	/**
	 * 物理删除指定文档
	 * @param userId	操作用户
	 * @param dID	文档的DID
	 */
	public void physicsDelete(String userId, String dID);

	/**
	 * 恢复已删除的文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void recover(UploadUserDto user, String dID);

	/**
	 * 订阅指定的文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void subscription(UploadUserDto user, String dID);

	/**
	 * 取消订阅指定文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void unSubscription(UploadUserDto user, String dID);

	/**
	 * 收藏指定文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void collection(UploadUserDto user, String dID);

	/**
	 * 取消收藏指定文档
	 * @param user	操作用户
	 * @param dID	文档的DID
	 */
	public void unCollection(UploadUserDto user, String dID);

	/**
	 * 获取和文档相关的所有信息
	 * @param user
	 * @param docId
	 * @return
	 * @throws BasException
	 */
	public DocModel getFullDocModelBydID(UploadUserDto user, String docId, String revId)throws BasException;
}
